/*
 * PAM S/Key module for access checking
 *
 * If you want to allow both s/key and plaintext passwords
 * based on the skey.access configuration mechanism, insert
 * this right after pam_skey.so
 *
 * Copyright (C) 1999, Olaf Kirch <okir@caldera.de>
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <syslog.h>
#include <fcntl.h>
#include <ctype.h>

#include <security/_pam_macros.h>
#include <security/pam_modules.h>

#ifndef LINUX_PAM
#include <security/pam_appl.h>
#endif  /* LINUX_PAM */

#include "skey.h"

static int	_skey_checkaccess(pam_handle_t *, int, const char *);

PAM_EXTERN int
pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	const char	*logname;
	int		retval;

	retval = pam_get_user(pamh, &logname, "login:");

	if (retval != PAM_SUCCESS) {
		if (retval == PAM_CONV_AGAIN) {
			D(("get user: waiting for data"));
			return PAM_INCOMPLETE;
		}
		return retval;
	}
	if (logname == NULL) {
		D(("null user name"));
		return PAM_USER_UNKNOWN;
	}

	D(("checking access"));
	retval = _skey_checkaccess(pamh, 0, logname);
	if (retval == SKEY_ACCESS_NOFILE
	 || retval == SKEY_ACCESS_CONSOLE)
		return PAM_IGNORE;
	if (retval < 0)
		return PAM_SESSION_ERR;
	if (retval != 1)
		return PAM_AUTH_ERR;

	return PAM_SUCCESS;
}


PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	return PAM_IGNORE;
}

static int
_skey_checkaccess(pam_handle_t *pamh, int ctrl, const char *logname)
{
	char	*host, *port;

	if (pam_get_item(pamh, PAM_RHOST, (const void **) &host) != PAM_SUCCESS)
		host = 0;
	if (pam_get_item(pamh, PAM_TTY, (const void **) &port) != PAM_SUCCESS)
		port = 0;

	return skeyaccess(logname, host, port);
}
