/*
 * skey.h header file
 *
 */

#ifndef SKEY_H
#define SKEY_H

#include <stdio.h>

/* Server-side data structure for reading keys file during login */
#define SKEY_BUFSIZE	256
struct skey {
	FILE *			keyfile;
	FILE *			lockfile;
	char			buf[SKEY_BUFSIZE];
	const char *		logname;
	int			n;
	const char *		seed;
	const unsigned char *	key;
	unsigned char		keybuf[8];
};

/* Client-side structure for scanning data stream for challenge */
struct mc {
	char		buf[256];
	int		skip;
	int		cnt;
};

extern void	f(unsigned char *);
extern int	keycrunch(unsigned char result[8],
			  const char *seed, const char *passwd);
extern int	skeychallenge(struct skey *, char *);
extern int	skeyverify(struct skey *mp, const char *response);
extern int	skeysetkey(struct skey *mp, const unsigned char *key);
extern int	skeyupdate(const struct skey *mp, const char *key);
extern int	skeylookup(struct skey *, const char *);
extern int	skeyinfo(struct skey *, const char *, char *);
extern void	skeyclose(struct skey *);

/* Access control */
extern int	skeyaccess(const char *, const char *, const char *);
/* Return values: */
enum {
	SKEY_ACCESS_PASSWDOK = 1,
	SKEY_ACCESS_SKEY_ONLY = 0,
	SKEY_ACCESS_EINVAL = -1,
	SKEY_ACCESS_NOFILE = -2,
	SKEY_ACCESS_SYNTAXERR = -3,
	SKEY_ACCESS_CONSOLE = -4,
};

/* Internal */
extern char *	btoe(char *, const unsigned char *);
extern char *	put8(char *out, const unsigned char *s);
extern int	etob(unsigned char *out, const char *e);
extern int	atob8(unsigned char *, const char *);
extern int	btoa8(char *, const unsigned char *);
extern void	rip(char *);
extern void	get_random_digits(char *, unsigned int);

#endif /* SKEY_H */
